--------------------------------------------------------------------------------
-- SubModule FIFOPipeIn
-- Created   8/28/2006 2:16:56 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity FIFOPipeIn is port
   (
     ti_clk            : in    std_logic;
     ti_control        : in    std_logic_vector(12 downto 0);
     ti_data           : in    std_logic_vector(15 downto 0);
     ep_status         : out   std_logic_vector(3 downto 0);
     ep_reset          : in    std_logic;
     ep_clk            : in    std_logic;
     ep_read           : in    std_logic;
     ep_full           : out   std_logic;
     ep_empty          : out   std_logic;
     ep_error          : out   std_logic;
     ep_addr           : in    std_logic_vector(7 downto 0);
     ep_dataout        : out   std_logic_vector(31 downto 0);
     ep_enable         : in    std_logic
   );
end FIFOPipeIn;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of FIFOPipeIn is

-- Component Declarations
    component okPipeIn port (
        ti_clk      : in    std_logic;
        ti_control  : in    std_logic_vector(12 downto 0);
        ti_data     : in    std_logic_vector(15 downto 0);
        ep_addr     : in    std_logic_vector(7 downto 0);
        ep_dataout  : out   std_logic_vector(15 downto 0);
        ep_write    : out   std_logic);
    end component;

    component fifo_read port (
        din         : in    std_logic_vector(15 downto 0);
        rd_clk      : in    std_logic;
        rd_en       : in    std_logic;
        rst         : in    std_logic;
        wr_clk      : in    std_logic;
        wr_en       : in    std_logic;
        dout        : out   std_logic_vector(31 downto 0);
        empty       : out   std_logic;
        full        : out   std_logic;
        overflow    : out   std_logic;
        rd_data_count: out  std_logic_vector(3 downto 0);
        underflow   : out   std_logic);
    end component;

-- Signal Declarations
   signal data             : std_logic_vector(15 downto 0);
   signal rden             : std_logic;
   signal wren             : std_logic;
   signal overflow         : std_logic;
   signal underflow        : std_logic;

begin
   wren <= ep_read when ep_enable = '1' else '0';

   pipe : okPipeIn port map (
      ti_clk => ti_clk,
      ti_control => ti_control,
      ti_data => ti_data,
      ep_addr => ep_addr,
      ep_dataout => data,
      ep_write => rden);

   fifo : fifo_read port map (
      wr_clk => ti_clk,
      wr_en => rden,
      din => data,
      rd_clk => ep_clk,
      rd_en => wren,
      dout => ep_dataout,
      rst => ep_reset,
      empty => ep_empty,
      full => ep_full,
      rd_data_count => ep_status,
      overflow => overflow,
      underflow => underflow);

   ep_error <= '0' when ep_reset = '1' else
               '1' when overflow = '1' or underflow = '1';

end Structure;
--------------------------------------------------------------------------------
