--------------------------------------------------------------------------------
-- SubModule IIR_ADC2
-- Created   4/12/2005 5:51:13 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity IIR_ADC2 is port
   (
     ADCA       : in    std_logic_vector(17 downto 0);  -- AD7679
     ADCB       : in    std_logic_vector(17 downto 0);  -- AD7679
     L          : in    std_logic;  -- busy signal of ADC
     O          : out   std_logic_vector(31 downto 0);  -- sign extended ADC value
     OVF        : out   std_logic   -- overflow
   );
end IIR_ADC2;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of IIR_ADC2 is

   signal extA : std_logic_vector(31 downto 0);
   signal extB : std_logic_vector(31 downto 0);
   signal sum : signed(31 downto 0) := (others => '0');

begin
   -- sign extend and shift ADC value into 32 bit word
   load: process (L) is
   begin
      if rising_edge(L) then
         extA(7 downto 0) <= (others => '0');
         extA(25 downto 8) <= ADCA;
         extA(31 downto 26) <= (others => ADCA(17));
         extB(7 downto 0) <= (others => '0');
         extB(25 downto 8) <= ADCB;
         extB(31 downto 26) <= (others => ADCB(17));
      end if;
   end process load;

   -- add all 4 ADC values
   sum <= signed(extA) + signed(extB);
   O <= std_logic_vector(sum);

   -- check overflow on ADC input
   overflow: process (L) is
   begin
      if rising_edge (L) then
         if (ADCA(17 downto 6) = B"011111111111") or
            (ADCA(17 downto 6) = B"100000000000") or
            (ADCB(17 downto 6) = B"011111111111") or
            (ADCB(17 downto 6) = B"100000000000") then
            OVF <= '1';
         else
            OVF <= '0';
         end if;
      end if;
   end process overflow;

end Structure;
--------------------------------------------------------------------------------
